uniform float outlineWidth;

uniform mat4 modelViewMatrix;
uniform mat4 modelViewProjectionMatrix;
uniform mat4 textureMatrix;

uniform vec4 additionalColor;
uniform vec4 ambientColor;

uniform float alpha;
uniform float shininess;
uniform bool useColors;

uniform float fogStart;
uniform float fogEnd;
uniform vec3 fogColor;

uniform int lightCount;

uniform vec3 lightPositions[8];
uniform vec3 diffuseColors[8];
uniform vec3 specularColors[8];
uniform float attenuation[8];

attribute vec4 position;
attribute vec3 normal;
attribute vec4 color;
attribute vec2 texture0;
attribute vec2 texture1;
attribute vec2 texture2;
attribute vec2 texture3;

varying vec2 texCoord[4];
varying vec4 vertexColor;
varying vec3 fogVertexColor;
varying float fogWeight;

const vec4 WHITE = vec4(1,1,1,1);

void main() {

	texCoord[0] = (textureMatrix * vec4(texture0, 0, 1)).xy;	
	texCoord[1] = texture1;
	texCoord[2] = texture2;
	texCoord[3] = texture3;

	vec4 vertexPos = modelViewMatrix * vec4(vec3(position) + (outlineWidth * normal), position.w);
	vertexColor = ambientColor + additionalColor;
	
	if (fogStart != -1.0) {
		fogWeight = clamp((-vertexPos.z - fogStart) / (fogEnd - fogStart), 0.0, 1.0);
		fogVertexColor = fogColor * fogWeight;
	} else {
		fogWeight = -1.0;
	}
	
	vertexColor=vec4(min(WHITE, vertexColor).xyz, alpha);
	
	if (useColors) {
		vertexColor *= color;
	}
	
	gl_Position = modelViewProjectionMatrix * vec4(vec3(position) + (outlineWidth * normal), position.w);
}